﻿using System.Drawing;
using OpenCvSharp;

namespace LibOpenCV.Methods
{
    public partial class OpenCV
    {
        private readonly CvColor[] colors = new CvColor[]{
                new CvColor(0,0,255),
                new CvColor(0,128,255),
                new CvColor(0,255,255),
                new CvColor(0,255,0),
                new CvColor(255,128,0),
                new CvColor(255,255,0),
                new CvColor(255,0,0),
                new CvColor(255,0,255),
            };

        public Bitmap FaceDetect( Bitmap Orignal )
        {
            // BitmapからIplImageに変換
            using( var gray = new IplImage( Orignal.Width, Orignal.Height, BitDepth.U8, 1 ) )
            using( var ipl = new IplImage( Orignal.Width, Orignal.Height, BitDepth.U8, 3 ) ) {
                ipl.CopyFrom( new Bitmap( Orignal ) );

                // 顔検出用の画像生成
                Cv.CvtColor( ipl, gray, ColorConversion.BgrToGray );

                // 
                using( var cascade = CvHaarClassifierCascade.FromFile( "haarcascade_frontalface_alt.xml" ) )
                using( var storage = new CvMemStorage() ) {
                    storage.Clear();

                    // 顔の検出
                    var faces = Cv.HaarDetectObjects( gray, cascade, storage, 1.14, 2, 0, new CvSize( 3, 3 ) );

                    // 検出箇所に丸を付ける
                    for( int i = 0; i < faces.Total; i++ ) {
                        var r = faces[i].Value.Rect;
                        var center = new CvPoint
                        {
                            X = Cv.Round( r.X + r.Width * 0.5 ),
                            Y = Cv.Round( r.Y + r.Height * 0.5 )
                        };
                        var radius = Cv.Round( (r.Width + r.Height) * 0.25 );
                        ipl.Circle( center, radius, colors[i % 8], 3, LineType.AntiAlias, 0 );
                    }
                }

                return ipl.ToBitmap();
            }
        }
    }
}
